// ----------------------------------------------------------------------------
// Sample Code for driving a piezo-mounted mirror, reading a Light Dependent 
// Resistor Module and printing the data to serial. This sketch is part of
// the CERN - S'Cool LAB 3D printed Laserlab project. More information to be
// found on the S'Cool LAB website: https://scoollab.web.cern.ch/laserlab3D
// 
// This project has been tested on an Arduino UNO microcontroller board.
//
// Copyright 2020, Fabian Bernstein CERN - S'Cool LAB 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights to
// use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is furnished to do
// so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
// SOFTWARE. 
// ----------------------------------------------------------------------------

// INPUT AND OUTPUT PINS ------------------------------------------------------
const int piezoPin = 5;         // Pin going to the piezo via the low pass filter.
                                // Check schematics in assembly instruction for details on the wiring.
const int analogPin = A0;       // Pin where the anlog output from the LDR is connected to.


// PROGRAM VARIABLES ----------------------------------------------------------
const char sDelimiter = ' ';    // Serial Plotter delimiter. Seperates Light Dependent Resistor (LDR) Value, Piezo Value, and Piezo Voltage 
unsigned int AnalogValue;       // Variable to store LDR value

unsigned long time_now = 0;     // Current time
const int waitTime = 2000;      // Time interval between the voltage ramps of the piezo
const int delayTime = 50;       // Waiting time between measurements of the LDR

int piezoValue = 1;             // Variable for driving the piezo by means of analogWrite. Range 1 - 251.
float piezoVoltage = 0;         // Voltage applied to the piezo calculated as piezoValue *(5/255) 


// SETUP----------------------------------------------------------------------------------
void setup() {
  Serial.begin(9600);
  pinMode(piezoPin, OUTPUT);
}

// START OF MAIN LOOP  --------------------------------------------------------------------
void loop() {
  piezoRamp(); //Run voltage ramp
  time_now = millis(); //Get current timestamp
  while (millis() < time_now + waitTime) { //Wait until next ramp is executed
    processLDR();
    delay(delayTime);
  }
}

// READ LDR AND SEND DATA TO SERIAL---------------------------------------------------------
void processLDR()
{
  AnalogValue = analogRead(analogPin); // Read LDR Value
  piezoVoltage = (float(piezoValue) / 255) * 5; //Calculate current output voltage to the piezo; 
                                                //conversion of piezoValue from int to float necessary for division

  Serial.print(AnalogValue); // Pass LDR Value to Serial Port
  Serial.print(sDelimiter); // Delimiter necessary for Serial Plotter

  Serial.print(piezoValue); // Pass piezo value (Range 1 - 251) to Serial Port
  Serial.print(sDelimiter); // Delimiter necessary for Serial Plotter
  
  Serial.println(piezoVoltage); // Pass piezo voltage (Range 0 V - 5 V) to Serial Port

}

// RUN THE VOLTAGE RAMP ON THE PIEZO-----------------------------------------------------------
// The voltage applied to the piezo will ramp up from 0 V to 4,92 V and remain at that voltage 
// level for half the waiting time. Then the voltage will return to 0 V and will stay at 0 V for
// the duration of the waiting time. The idea is to wait until the oscillation of
// the mirror has stopped. After that, the cycle will start anew.
  
void piezoRamp()
{
  for (piezoValue = 1; piezoValue <= 255; piezoValue = piezoValue + 10) {
    analogWrite(piezoPin, piezoValue);
    processLDR();
    delay(delayTime);
  }
  piezoValue = piezoValue - 10; //Set piezoValue to last sent value 
  time_now = millis(); //Get current timestamp
  while (millis() < time_now + waitTime / 2) { //Wait for half the waiting time before resetting piezo voltage to zero
    processLDR();
    delay(delayTime);
  }
  piezoValue = 0; //Reset piezo to zero position
  analogWrite(piezoPin, piezoValue); 

}
